#!/usr/bin/php
<?php
/*
* Switch/Router Config Wizard
* Copyright (c) 2008-2016 Nagios Enterprises, LLC. All rights reserved.
*
* Use this utility to convert any mrtg configuration file created prior to Switch Configwizard 2.3.0 to the new style configuration files
* After you run this utility for each configuration file
* 
* Run (something like) the following command to upgrade all legacy mrtg configuration files:
* find /etc/mrtg/conf.d/ -regex ".*.cfg" | grep -v "_[0-9]*.cfg" | awk -F/ ' { print $5 } ' | sed 's/.cfg//' | xargs /usr/local/nagiosxi/html/includes/configwizards/switch/convert_legacy_mrtg_cfg.php
*/

require_once('switch.inc.php');
echo "\n";

function print_to_stderr_and_exit($errors) {
	$stderr = fopen('php://stderr', 'w+');
	fwrite($stderr, $errors);
	fclose($stderr);
	exit(1);
}

function usage($arg0) {
	return 
		"You have an error in your syntax!\n" .
		"Convert Legacy MRTG Configuration script\n" .
		"To use,\n\t{$arg0} [address] [address]2 .. [address]N\n\n" .
		"Where [address] is the hostaddress used to build the legacy MRTG configuration file\n" .
		"e.g.: (/etc/mrtg/conf.d/[address].cfg)\n\n" .
		"You may enter an unlimited number of addresses.\n\n";
}

// bail out if we don't have at least the right amount of arguments
if (count($argv) < 2) {
	print_to_stderr_and_exit(usage($argv[0]));
}

// cycle through the arguments
$arg_counter = 1;
for ($arg_counter = 1; $arg_counter < count($argv); $arg_counter++) {

	$address = $argv[$arg_counter];
	$file = @switch_configwizard_get_legacy_mrtg_cfg_file($address);

	echo "Working on {$address}\n";

	// check if file exists
	if (!@file_exists($file)) {
		print_to_stderr_and_exit("Can't find {$file}\n\n");
	}

	// check if we have readable permissions on file
	if (!@is_readable($file)) {
		print_to_stderr_and_exit("No permission to read {$file}\n\n");
	}

	// convert the file!
	$converted = @switch_configwizard_convert_legacy_file($address);

	if ($converted == true) {
		echo "Conversion of {$file} was successful.\n\n";
	} else {
		print_to_stderr_and_exit($converted);
	}
}

?>